import os,sys
path=os.path.join(request.folder,'modules')
if not path in sys.path: sys.path.append(path)
from openid.consumer import consumer
from openid.consumer.discover import DiscoveryFailure
from openid.extensions import pape, sreg
from openid.yadis.constants import *
from openid.server.trustroot import RP_RETURN_TO_URL_TYPE
from openid.store.filestore import FileOpenIDStore

# Compute the trust root and return URL values to build the
# redirect information, here is hardcoded
trust_root = 'http://127.0.0.1:8000/openid/client/startOpenID'
return_to = 'http://127.0.0.1:8000/openid/client/finishOpenID'

PAPE_POLICIES = [
    'AUTH_PHISHING_RESISTANT',
    'AUTH_MULTI_FACTOR',
    'AUTH_MULTI_FACTOR_PHYSICAL',
    ]

# List of (name, uri) for use in generating the request form.
POLICY_PAIRS = [(p, getattr(pape, p)) for p in PAPE_POLICIES]

def __getConsumer():
    """
    Get a Consumer object to perform OpenID authentication.
    """
    path=os.path.join(request.folder,'databases')
    return consumer.Consumer(session, FileOpenIDStore(path))

def startOpenID():
    """
    Start the OpenID authentication process.  Renders an
    authentication form and accepts its POST.
    * Renders an error message if OpenID cannot be initiated
    * Requests some Simple Registration data using the OpenID
      library's Simple Registration machinery
    * Generates the appropriate trust root and return URL values for
      this application (tweak where appropriate)
    * Generates the appropriate redirect based on the OpenID protocol
      version.
    """
    if request.post_vars:
        # Start OpenID authentication.
        openid_url = request.post_vars.openid_identifier
        c = __getConsumer()
        error = None
        try:
            auth_request = c.begin(openid_url)
        except DiscoveryFailure, e:
            # Some other protocol-level failure occurred.
            session.flash="OpenID discovery error: %s" % str(e)
            redirect(URL(r=request,f='startOpenID'))
        # Add Simple Registration request information.  Some fields
        # are optional, some are required.  It's possible that the
        # server doesn't support sreg or won't return any of the
        # fields.
        sreg_request = sreg.SRegRequest(optional=['email', 'nickname'],
                                        required=['dob'])
        auth_request.addExtension(sreg_request)

        # Add PAPE request information.  We'll ask for
        # phishing-resistant auth and display any policies we get in
        # the response.
        requested_policies = []
        policy_prefix = 'policy_'
        for k, v in request.post_vars.iteritems():
            if k.startswith(policy_prefix):
                policy_attr = k[len(policy_prefix):]
                if policy_attr in PAPE_POLICIES:
                    requested_policies.append(getattr(pape, policy_attr))

        if requested_policies:
            pape_request = pape.Request(requested_policies)
            auth_request.addExtension(pape_request)

        url = auth_request.redirectURL(trust_root, return_to)
        return redirect(url)
    return dict(message='I am here')

def finishOpenID():
    """
    Finish the OpenID authentication process.  Invoke the OpenID
    library with the response from the OpenID server and render a page
    detailing the result.
    """
    result = {}

    # Because the object containing the query parameters is a
    # MultiValueDict and the OpenID library doesn't allow that, we'll
    # convert it to a normal dict.

    # OpenID 2 can send arguments as either POST body or GET query
    # parameters.
    request_args = dict(request.get_vars)
    if request.post_vars:
        request_args.update(dict(request.post_vars))

    if request_args:
        c = __getConsumer()

        # Get a response object indicating the result of the OpenID
        # protocol.
        return_to = URL(r=request,f='finishOpenID')
        openid_response = c.complete(request_args, return_to)

        # Get a Simple Registration openid_response object if openid_response
        # information was included in the OpenID openid_response.
        sreg_response = {}
        if openid_response.status == consumer.SUCCESS:
            sreg_response = sreg.SRegResponse.fromSuccessResponse(openid_response)

        # Get a PAPE openid_response object if openid_response information was
        # included in the OpenID openid_response.
        pape_response = None
        if openid_response.status == consumer.SUCCESS:
            pape_response = pape.Response.fromSuccessResponse(openid_response)

        # Map different consumer status codes to template contexts.
        results = {
            consumer.CANCEL:
            {'message': 'OpenID authentication cancelled.'},

            consumer.FAILURE:
            {'error': 'OpenID authentication failed.'},

            consumer.SUCCESS:
            {'url': openid_response.getDisplayIdentifier(),
             'sreg': sreg_response.items(),
             'pape': pape_response},
            }

        result = results[openid_response.status]

        if isinstance(openid_response, consumer.FailureResponse):
            # In a real application, this information should be
            # written to a log for debugging/tracking OpenID
            # authentication failures. In general, the messages are
            # not user-friendly, but intended for developers.
            result['failure_reason'] = openid_response.message

    return dict(message='something heppened:'+str(result))

